# standard lib
import logging
import os
import re
import shutil

# other 3rd party
import pytest
from click.testing import CliRunner

# MkDocs
from mkdocs.__main__ import build_command

# ##################################
# ######## Globals #################
# ##################################

# custom log level to get plugin info messages
logging.basicConfig(level=logging.INFO)

# ##################################
# ########## Helpers ###############
# ##################################


def setup_clean_mkdocs_folder(
    mkdocs_yml_path: str, output_path: str, docs_path: str = "tests/fixtures/docs"
):
    """
    Sets up a clean mkdocs directory
    outputpath/testproject
    ├── docs/
    └── mkdocs.yml
    Args:
        mkdocs_yml_path (Path): Path of mkdocs.yml file to use
        output_path (Path): Path of folder in which to create mkdocs project
    Returns:
        testproject_path (Path): Path to test project
    """

    testproject_path = output_path / "testproject"

    # Create empty 'testproject' folder
    if os.path.exists(str(testproject_path)):
        logging.warning(
            """This command does not work on windows.
        Refactor your test to use setup_clean_mkdocs_folder() only once"""
        )
        shutil.rmtree(str(testproject_path))

    # Copy correct mkdocs.yml file and our test 'docs/'
    shutil.copytree(docs_path, str(testproject_path / docs_path.split("/")[-1]))

    shutil.copyfile(mkdocs_yml_path, str(testproject_path / "mkdocs.yml"))

    return testproject_path


def build_docs_setup(testproject_path: str):
    """
    Runs the `mkdocs build` command
    Args:
        testproject_path (Path): Path to test project
    Returns:
        command: Object with results of command
    """

    # TODO: Try specifying path in CliRunner, this function could be one-liner
    cwd = os.getcwd()
    os.chdir(str(testproject_path))

    try:
        runner = CliRunner()
        run = runner.invoke(build_command)
        os.chdir(cwd)
        return run
    except:
        os.chdir(cwd)
        raise


def validate_build(testproject_path: str, plugin_config: dict = {}):
    """
    Validates a build from a testproject
    Args:
        testproject_path (Path): Path to test project
    """
    assert os.path.exists(str(testproject_path / "site"))

    # Make sure index file exists
    index_file = testproject_path / "site/index.html"
    assert index_file.exists(), "%s does not exist" % index_file


def validate_mkdocs_file(
    temp_path: str, mkdocs_yml_file: str, docs_path: str = "tests/fixtures/docs"
):
    """
    Creates a clean mkdocs project
    for a mkdocs YML file, builds and validates it.
    Args:
        temp_path (PosixPath): Path to temporary folder
        mkdocs_yml_file (PosixPath): Path to mkdocs.yml file
    """
    testproject_path = setup_clean_mkdocs_folder(
        mkdocs_yml_path=mkdocs_yml_file, output_path=temp_path, docs_path=docs_path
    )
    result = build_docs_setup(
        testproject_path,
    )
    assert result.exit_code == 0, "'mkdocs build' command failed"

    # validate build with locale retrieved from mkdocs config file
    validate_build(testproject_path)

    return testproject_path


def validate_static(html_content: str, path: str = "", exist: bool = True):
    """
    Validate glightbox.min.css and glightbox.min.js have been loaded or not
    """
    assert exist == (
        re.search(
            rf'<link href="{re.escape(path)}assets\/stylesheets\/glightbox\.min\.css" rel="stylesheet"\/>',
            html_content,
        )
        is not None
    )
    assert exist == (
        re.search(
            rf'<script src="{re.escape(path)}assets\/javascripts\/glightbox\.min\.js"><\/script>',
            html_content,
        )
        is not None
    )


def validate_script(html_content: str, exist: bool = True):
    """
    Validate GLightbox have been initialized or not
    """
    assert exist == (
        re.search(
            r"const lightbox = GLightbox\((.*)\);",
            html_content,
        )
        is not None
    )


EMOJI_LIST = ["emojione", "gemoji", "twemoji"]

# ##################################
# ########### Tests ################
# ##################################


def test_basic(tmp_path):
    """
    Minimal sample
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    assert re.search(
        r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
        contents,
    )


def test_material(tmp_path):
    """
    Integrate with Material for MkDocs
    """
    mkdocs_file = "mkdocs-material.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    assert re.search(
        r"document\$\.subscribe\(\(\) => { lightbox.reload\(\) }\);",
        contents,
    )
    assert re.search(
        r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
        contents,
    )


def test_material_instant(tmp_path):
    """
    Integrate with Material for MkDocs
    """
    mkdocs_file = "mkdocs-material-instant.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    assert re.search(
        r"document\$\.subscribe\(\(\) => { lightbox.reload\(\) }\);",
        contents,
    )
    assert re.search(
        r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
        contents,
    )


def test_use_directory_urls(tmp_path):
    """
    Compatible with use_directory_urls is false or with --use-directory-urls and --use-directory-urls as args
    https://www.mkdocs.org/user-guide/configuration/#use_directory_urls
    https://www.mkdocs.org/user-guide/cli/
    """
    mkdocs_file = "mkdocs-target-file.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/sub_dir/page_in_sub_dir.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path=path)
    validate_script(contents)
    assert re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png".*?>\s*<img.*?src="\.\.\/img\.png".*?\/><\/a>',
        contents,
    )


def test_disable_by_page(tmp_path):
    """
    Disable by page with page meta
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/disable_by_page/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents, exist=False)
    validate_script(contents, exist=False)
    assert (
        re.search(
            r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
            contents,
        )
        is None
    )


def test_disable_by_image(tmp_path):
    """
    Disable by the image with image custom class or predefined class
    """
    mkdocs_file = "mkdocs-disable-by-image.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/disable_by_image/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path=path)
    validate_script(contents)
    assert re.search(
        rf'<p><img.*?class="off-glb".*?src="{re.escape(path)}img\.png".*?\/><\/p>',
        contents,
    )
    assert re.search(
        rf'<p><img.*?class="skip-lightbox".*?src="{re.escape(path)}img\.png".*?\/><\/p>',
        contents,
    )
    assert re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png".*?><img.*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )


@pytest.mark.parametrize("emoji_name", EMOJI_LIST)
def test_disable_with_emoji(emoji_name, tmp_path):
    """
    Disable when the image with emoji class name(defined in PyMdown Extensions): emojione, gemoji, twemoji
    https://facelessuser.github.io/pymdown-extensions/extensions/emoji/
    """
    mkdocs_file = f"mkdocs-material-{emoji_name}.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/emoji/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents, path="../")
    validate_script(contents)
    assert re.search(
        rf'<p><img.*?class="{re.escape(emoji_name)}".*?\/>.*?<\/p>',
        contents,
    )


def test_url(tmp_path):
    """
    Compatible with URL
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/url/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents, path="../")
    validate_script(contents)
    image_url = re.escape("https://dummyimage.com/600x400/bdc3c7/fff.png")
    assert re.search(
        rf'<a class="glightbox".*?href="{image_url}".*?><img.*?src="{image_url}".*?\/><\/a>',
        contents,
    )


def test_default_options(tmp_path):
    """
    Validate GLightbox default options
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/images/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path)
    validate_script(contents)
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png.*?"><img.*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )

    # validate position
    regex_obj = re.search(
        r"const lightbox = GLightbox\((.*)\);",
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    options = [
        '"touchNavigation": true',
        '"loop": false',
        '"zoomable": true',
        '"draggable": true',
        '"openEffect": "zoom"',
        '"closeEffect": "zoom"',
        '"slideEffect": "slide"',
    ]
    for option in options:
        assert option in text


def test_options(tmp_path):
    """
    Validate GLightbox options setting through plugins
    """
    mkdocs_file = "mkdocs-options.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/images/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path)
    validate_script(contents)
    # validate override style
    assert ".gslide-image img { background: none; }" in contents
    assert (
        """.glightbox-clean .gslide-media {
        -webkit-box-shadow: none;
        box-shadow: none;
    }"""
        in contents
    )
    # validate slide options
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img.*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-height="60%"' in text
    assert 'data-width="80%"' in text
    assert 'data-desc-position="right"' in text

    # validate GLightbox options
    regex_obj = re.search(
        r"const lightbox = GLightbox\((.*)\);",
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    options = [
        '"touchNavigation": false',
        '"loop": true',
        '"zoomable": false',
        '"draggable": false',
        '"openEffect": "fade"',
        '"closeEffect": "fade"',
        '"slideEffect": "fade"',
    ]
    for option in options:
        assert option in text


def test_gallery(tmp_path):
    """
    Validate gallery
    """
    mkdocs_file = "mkdocs-material.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/gallery/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path=path)
    validate_script(contents)
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img alt="image-a".*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-gallery="1"' in text

    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img alt="image-b".*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-gallery="1"' in text

    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}another-img\.png"(.*?)><img alt="image-c".*?src="{re.escape(path)}another-img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-gallery="2"' in text

    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}another-img\.png"(.*?)><img alt="image-d".*?src="{re.escape(path)}another-img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-gallery="2"' in text


def test_caption(tmp_path):
    """
    Validate captions feature
    """
    mkdocs_file = "mkdocs-material.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/caption/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path=path)
    validate_script(contents)

    # validate title and description
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img alt="image-default".*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-description="data-description"' in text
    assert 'data-title="data-title"' in text

    # validate position
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img alt="image-right".*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-desc-position="right"' in text

    # validate compatible with figure
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img alt="image-figure".*?src="{re.escape(path)}img\.png".*?\/><\/a><\/p>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-description="data-description"' in text
    assert 'data-title="data-title"' in text


def test_auto_caption_by_page(tmp_path):
    """
    Validate auto captions with image tag alt by page
    """
    mkdocs_file = "mkdocs-material.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/auto_caption/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path=path)
    validate_script(contents)
    # validate title and description
    regex_obj = re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png"(.*?)><img.*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-title="alt as caption"' in text


def test_auto_caption(tmp_path):
    """
    Validate auto captions with image tag alt for all page
    """
    mkdocs_file = "mkdocs-auto-caption.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    regex_obj = re.search(
        r'<a class="glightbox".*?href="img\.png"(.*?)><img.*?src="img\.png".*?\/><\/a>',
        contents,
    )
    assert regex_obj
    text = regex_obj.group(1)
    assert 'data-title="image"' in text


def test_material_template(tmp_path):
    """
    Compatible with template
    """
    mkdocs_file = "mkdocs-material-template.yml"
    testproject_path = validate_mkdocs_file(
        tmp_path,
        f"tests/fixtures/{mkdocs_file}",
        docs_path="tests/fixtures/template_docs",
    )
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    assert re.search(
        r"document\$\.subscribe\(\(\) => { lightbox.reload\(\) }\);",
        contents,
    )
    assert re.search(
        r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
        contents,
    )


def test_site_url(tmp_path):
    """
    Compatible with the site with path prefix
    """
    mkdocs_file = "mkdocs-site-url.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    assert re.search(
        r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
        contents,
    )

    file = testproject_path / "site/images/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path)
    validate_script(contents)
    assert re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png".*?><img.*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )


def test_static(tmp_path):
    """
    Validate static files
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    assert os.path.exists(
        os.path.join(testproject_path, "site/assets/stylesheets/glightbox.min.css")
    )
    assert os.path.exists(
        os.path.join(testproject_path, "site/assets/javascripts/glightbox.min.js")
    )


def test_image_in_anchor(tmp_path):
    """
    Disable when image in an anchor tag
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/image_in_anchor/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path)
    validate_script(contents)
    assert (
        re.search(
            rf'<a class="glightbox".*?href="{re.escape(path)}img\.png".*?><img.*?src="{re.escape(path)}img\.png".*?\/><\/a>',
            contents,
        )
        is None
    )


def test_image_without_ext(tmp_path):
    """
    Image without extension
    """
    mkdocs_file = "mkdocs.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/without_ext/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path)
    validate_script(contents)
    assert re.search(
        r'<a class="glightbox".*?href="https://picsum\.photos/1200/800".*?data-type="image".*?><img.*?src="https://picsum\.photos/1200/800".*?\/><\/a>',
        contents,
    )


def test_error(tmp_path):
    """
    Wrapping for error
    """
    mkdocs_file = "mkdocs-error.yml"
    testproject_path = validate_mkdocs_file(
        tmp_path,
        f"tests/fixtures/{mkdocs_file}",
        docs_path="tests/fixtures/error_docs",
    )
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)


def test_privacy(tmp_path):
    """
    Compatible with material privacy plugin
    """
    mkdocs_file = "mkdocs-material-privacy.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/url/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents, path="../")
    validate_script(contents)
    image_url = re.escape("../assets/external/dummyimage.com/600x400/bdc3c7/fff.png")
    assert re.search(
        rf'<a class="glightbox"(?!.*href=).*?><img.*?src="{image_url}".*?><\/a>',
        contents,
    )
    patch_script = re.escape(
        "document.querySelectorAll('.glightbox').forEach(function(element) {"
    )
    assert re.search(
        rf"{patch_script}",
        contents,
    )


def test_enable_by_image(tmp_path):
    """
    Enable by the image with on-glb class
    """
    mkdocs_file = "mkdocs-material.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/enable_by_image/index.html"
    contents = file.read_text(encoding="utf8")
    path = "../"
    validate_static(contents, path=path)
    validate_script(contents)
    assert re.search(
        rf'<p><img alt="image" src="{re.escape(path)}img\.png" \/><\/p>',
        contents,
    )
    assert re.search(
        rf'<a class="glightbox".*?href="{re.escape(path)}img\.png".*?><img.*?class="on-glb".*?src="{re.escape(path)}img\.png".*?\/><\/a>',
        contents,
    )


def test_manual(tmp_path):
    """
    Manual mode
    """
    mkdocs_file = "mkdocs-manual.yml"
    testproject_path = validate_mkdocs_file(tmp_path, f"tests/fixtures/{mkdocs_file}")
    file = testproject_path / "site/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents)
    validate_script(contents)
    assert (
        re.search(
            r'<a class="glightbox".*?href="img\.png".*?>\s*<img.*?src="img\.png".*?\/><\/a>',
            contents,
        )
        is None
    )

    file = testproject_path / "site/manual/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents, path="../")
    validate_script(contents)
    assert re.search(
        r'<a class="glightbox".*?href="..\/img\.png".*?>\s*<img.*?src="..\/img\.png".*?\/><\/a>',
        contents,
    )

    file = testproject_path / "site/manual_enable_by_page/index.html"
    contents = file.read_text(encoding="utf8")
    validate_static(contents, path="../")
    validate_script(contents)
    assert re.search(
        r'<a class="glightbox".*?href="..\/img\.png".*?>\s*<img.*?alt="image-a" src="..\/img\.png".*?\/><\/a>',
        contents,
    )

    assert (
        re.search(
            r'<a class="glightbox".*?href="..\/img\.png".*?>\s*<img.*?alt="image-b" src="..\/img\.png".*?\/><\/a>',
            contents,
        )
        is None
    )

    assert re.search(
        r'<a class="glightbox".*?href="..\/img\.png".*?>\s*<img.*?alt="image-c" src="..\/img\.png".*?\/><\/a>',
        contents,
    )



import json
import logging
import os
import re

from mkdocs import utils
from mkdocs.config import config_options
from mkdocs.plugins import BasePlugin

log = logging.getLogger(__name__)
base_path = os.path.dirname(os.path.abspath(__file__))


class LightboxPlugin(BasePlugin):
    """Add lightbox to MkDocs"""

    config_scheme = (
        ("touchNavigation", config_options.Type(bool, default=True)),
        ("loop", config_options.Type(bool, default=False)),
        ("effect", config_options.Choice(("zoom", "fade", "none"), default="zoom")),
        (
            "slide_effect",
            config_options.Choice(("slide", "zoom", "fade", "none"), default="slide"),
        ),
        ("width", config_options.Type(str, default="auto")),
        ("height", config_options.Type(str, default="auto")),
        ("zoomable", config_options.Type(bool, default=True)),
        ("draggable", config_options.Type(bool, default=True)),
        ("skip_classes", config_options.Type(list, default=[])),
        ("auto_caption", config_options.Type(bool, default=False)),
        (
            "caption_position",
            config_options.Choice(("bottom", "top", "left", "right"), default="bottom"),
        ),
        ("background", config_options.Type(str, default="white")),
        ("shadow", config_options.Type(bool, default=True)),
        ("manual", config_options.Type(bool, default=False)),
    )

    def on_config(self, config):
        self.using_material = config["theme"].name == "material"
        self.using_material_privacy = (
            self.using_material
            and "material/privacy" in config["plugins"]
            and config["plugins"]["material/privacy"].config.enabled
        )

    def on_post_page(self, output, page, config, **kwargs):
        """Add css link tag, javascript script tag, and javascript code to initialize GLightbox"""
        # skip page with meta glightbox is false
        if "glightbox" in page.meta and page.meta.get("glightbox", True) is False:
            return output

        # Define regular expressions for matching the relevant sections of the HTML code
        head_regex = re.compile(r"<head>(.*?)<\/head>", flags=re.DOTALL)
        body_regex = re.compile(r"<body(.*?)<\/body>", flags=re.DOTALL)

        # Modify the CSS link
        css_link = f'<link href="{utils.get_relative_url(utils.normalize_url("assets/stylesheets/glightbox.min.css"), page.url)}" rel="stylesheet"/>'
        output = head_regex.sub(f"<head>\\1 {css_link}</head>", output)

        # Modify the CSS patch
        css_patch = """
    html.glightbox-open { overflow: initial; height: 100%; }
    .gslide-title { margin-top: 0px; user-select: text; }
    .gslide-desc { color: #666; user-select: text; }"""
        css_patch += f"""
    .gslide-image img {{ background: {self.config['background']}; }}"""
        if not self.config["shadow"]:
            css_patch += """
    .glightbox-clean .gslide-media {
        -webkit-box-shadow: none;
        box-shadow: none;
    }"""
        if config["theme"].name == "material":
            css_patch += """
    .gscrollbar-fixer { padding-right: 15px; }
    .gdesc-inner { font-size: 0.75rem; }
    body[data-md-color-scheme="slate"] .gdesc-inner { background: var(--md-default-bg-color);}
    body[data-md-color-scheme="slate"] .gslide-title { color: var(--md-default-fg-color);}
    body[data-md-color-scheme="slate"] .gslide-desc { color: var(--md-default-fg-color);}"""
        output = head_regex.sub(f"<head>\\1<style>{css_patch}</style></head>", output)

        # Modify the JS script
        js_script = f'<script src="{utils.get_relative_url(utils.normalize_url("assets/javascripts/glightbox.min.js"), page.url)}"></script>'
        output = head_regex.sub(f"<head>\\1 {js_script}</head>", output)

        # Modify the JS code
        plugin_config = dict(self.config)
        lb_config = {
            k: plugin_config[k]
            for k in ["touchNavigation", "loop", "zoomable", "draggable"]
        }
        lb_config["openEffect"] = plugin_config.get("effect", "zoom")
        lb_config["closeEffect"] = plugin_config.get("effect", "zoom")
        lb_config["slideEffect"] = plugin_config.get("slide_effect", "slide")
        js_code = ""
        if self.using_material_privacy:
            js_code += """document.querySelectorAll('.glightbox').forEach(function(element) {
    try {
        var img = element.querySelector('img');
        if (img && img.src) {
            element.setAttribute('href', img.src);
        } else {
            console.log('No img element with src attribute found');
        }
    } catch (error) {
        console.log('Error:', error);
    }
});
"""
        js_code += f"const lightbox = GLightbox({json.dumps(lb_config)});\n"
        if self.using_material or "navigation.instant" in config["theme"].get(
            "features", []
        ):
            # support compatible with mkdocs-material Instant loading feature
            js_code += "document$.subscribe(() => { lightbox.reload() });\n"
        output = body_regex.sub(
            f'<body\\1<script id="init-glightbox">{js_code}</script></body>', output
        )

        return output

    def on_page_content(self, html, page, config, **kwargs):
        """Wrap img tag with anchor tag with glightbox class and attributes from config"""
        # skip page with meta glightbox is false
        if "glightbox" in page.meta and page.meta.get("glightbox", True) is False:
            return html
        plugin_config = {k: dict(self.config)[k] for k in ["width", "height"]}
        # skip emoji img with index as class name from pymdownx.emoji https://facelessuser.github.io/pymdown-extensions/extensions/emoji/
        skip_class = ["emojione", "twemoji", "gemoji"]
        # skip image with off-glb and specific class
        skip_class += ["off-glb"] + self.config["skip_classes"]

        # Use regex to find image tags that need to be wrapped with anchor tags and image tags already wrapped with anchor tags
        pattern = re.compile(
            r"<a\b[^>]*>(?:\s*<[^>]+>\s*)*<img\b[^>]*>(?:\s*<[^>]+>\s*)*</a>|<img(?P<attr>.*?)>"
        )
        html = pattern.sub(
            lambda match: self.wrap_img_with_anchor(
                match, plugin_config, skip_class, page.meta
            ),
            html,
        )

        return html

    def wrap_img_with_anchor(self, match, plugin_config, skip_class, meta):
        """Wrap image tags with anchor tags"""
        try:
            a_pattern = re.compile(r"<a(?P<attr>.*?)>")
            if a_pattern.match(match.group(0)):
                return match.group(0)

            img_tag = match.group(0)
            img_attr = match.group("attr")
            classes = re.findall(r'class="([^"]+)"', img_attr)
            classes = [c for match in classes for c in match.split()]

            if self.config["manual"] and meta.get("glightbox", None) is True:
                # with manual mode but enable glightbox in page meta
                if set(skip_class) & set(classes):
                    # skip image with off-glb and specific class
                    return img_tag
            elif meta.get("glightbox-manual", False) or self.config["manual"]:
                # disable by page meta or global config
                if "on-glb" not in classes:
                    # skip image without on-glb class
                    return img_tag
            elif set(skip_class) & set(classes):
                # skip image with off-glb and specific class
                return img_tag

            if self.using_material_privacy:
                # skip href attribute if using material privacy plugin, will be set by js code
                a_tag = '<a class="glightbox" data-type="image"'
            else:
                src = re.search(r"src=[\"\']([^\"\']+)", img_attr).group(1)
                a_tag = f'<a class="glightbox" href="{src}" data-type="image"'
            # setting data-width and data-height with plugin options
            for k, v in plugin_config.items():
                a_tag += f' data-{k}="{v}"'
            slide_options = [
                "title",
                "description",
                "caption-position",
                "gallery",
            ]
            for option in slide_options:
                attr = f"data-{option}"
                if attr == "data-title":
                    val = re.search(r"data-title=[\"]([^\"]+)", img_attr)
                    if self.config["auto_caption"] or (
                        "glightbox.auto_caption" in meta
                        and meta.get("glightbox.auto_caption", False) is True
                    ):
                        if val:
                            val = val.group(1)
                        else:
                            val = re.search(r"alt=[\"]([^\"]+)", img_attr)
                            val = val.group(1) if val else ""
                    else:
                        val = val.group(1) if val else ""
                elif attr == "data-caption-position":
                    val = re.search(r"data-caption-position=[\"]([^\"]+)", img_attr)
                    val = val.group(1) if val else self.config["caption_position"]
                else:
                    val = re.search(f'{attr}=["]([^"]+)', img_attr)
                    val = val.group(1) if val else ""

                # skip val is empty
                if val != "":
                    # convert data-caption-position to data-desc-position
                    if attr == "data-caption-position":
                        a_tag += f' data-desc-position="{val}"'
                    else:
                        a_tag += f' {attr}="{val}"'
            a_tag += f">{img_tag}</a>"
            return a_tag
        except Exception as e:
            log.warning(
                f"Error in wrapping img tag with anchor tag: {e} {match.group(0)}"
            )
            return match.group(0)

    def on_post_build(self, config, **kwargs):
        """Copy glightbox"s css and js files to assets directory"""

        output_base_path = os.path.join(config["site_dir"], "assets")
        css_path = os.path.join(output_base_path, "stylesheets")
        utils.copy_file(
            os.path.join(base_path, "glightbox", "glightbox.min.css"),
            os.path.join(css_path, "glightbox.min.css"),
        )
        js_path = os.path.join(output_base_path, "javascripts")
        utils.copy_file(
            os.path.join(base_path, "glightbox", "glightbox.min.js"),
            os.path.join(js_path, "glightbox.min.js"),
        )



from setuptools import find_packages, setup

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="mkdocs-glightbox",
    version="0.4.0",
    author="Blueswen",
    author_email="blueswen.tw@gmail.com",
    url="https://blueswen.github.io/mkdocs-glightbox",
    project_urls={
        "Source": "https://github.com/Blueswen/mkdocs-glightbox",
    },
    keywords=["mkdocs", "plugin", "lightbox"],
    packages=find_packages(),
    license="MIT",
    description="MkDocs plugin supports image lightbox with GLightbox.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    entry_points={
        "mkdocs.plugins": [
            "glightbox = mkdocs_glightbox.plugin:LightboxPlugin",
        ]
    },
)
